﻿/*
===================================================================================================
AUTHOR   : Ken
Function : Set Tags & Add Tags by List
Created  : 2022-12-07
Modified : 2023-10-07
Version  : v1.2.3
===================================================================================================
*/



// Set Tags & Add Tags by List.
// (c) 2022 Ken

// This is a script for Directory Opus.
// See https://www.gpsoft.com.au/DScripts/redirect.asp?page=scripts for development information.



// Called by Directory Opus to initialize the script
function OnInit(initData)
{
	initData.name = "Set Tags & Add Tags by List & Find Tags by List (Via Everything)";
	initData.version = "1.2.3";
	initData.copyright = "(c) 2022 Ken";
	initData.url = "https://resource.dopus.com/t/set-tags-add-tags-by-list-beta/43269";
	initData.desc = "Use the command below to assign it to a shortcut keys or buttons";
	initData.default_enable = true;
	initData.min_version = "13.0";
}


function OnAddCommands(addCmdData)
{
    var cmd = addCmdData.AddCommand();
    cmd.name = "atbl";
    cmd.method = "Onatbl";
    cmd.desc = "Add Tags by List";
    cmd.label = "atbl";

	var cmd2 = addCmdData.AddCommand();
	cmd2.name = "settag";
	cmd2.method = "OnSetTag";
	cmd2.desc = "Set Tags";
	cmd2.label = "settag";

    var cmd3 = addCmdData.AddCommand();
    cmd3.name = "fbtl";
    cmd3.method = "Onfbtl";
    cmd3.desc = "Find by Tags List";
    cmd3.label = "fbtl";

    cmd.template = "";
    cmd.hide = false;
    cmd.icon = 'script';
}


 // Add string trim() method
 String.prototype.trim = function(s){s=s||"\\s";return this.replace(new RegExp("^("+s+"){1,}\|("+s+"){1,}$","g"),"");}
 var fsu = DOpus.FSUtil;
 var ST = DOpus.Create.StringTools();
 var cmd = DOpus.Create().Command();


function OnSetTag(scriptCmdData) {

    var dlgEdit = scriptCmdData.func.Dlg();
    var tab = scriptCmdData.func.sourcetab;
    var cmd = scriptCmdData.func.command;
	if (!tab.selected.count) {
		DOpus.Notify("Set Tags", "Nothing selected!");
		return
	}

	InitTags(tab, true);

    dlgEdit.title = "Directory Opus";
    dlgEdit.template = "dlgSetTags";
    dlgEdit.detach = true;
    dlgEdit.Create();

	dlgEdit.Control("static").style = "b";
	dlgEdit.Control("static").label = 'Set Tags';                               // Topic
	dlgEdit.Control("static2").label = 'One tag per line, or enter new tags separated by commas.';   // Notes
	dlgEdit.Control("static3").label = Script.LoadImage("Label.png");           // Icon

    var ADS = tab.vars.Get('ADScheck');
	dlgEdit.Control("SetTags").value = ADS;                                     // Default value
	dlgEdit.Control("SetTags").SelectRange(0, -1);                              //Select Default value
	dlgEdit.AddHotkey("enterdlg", "ctrl+enter")                                 // Add Hotkeys

    dlgEdit.Show();

	var newTags;

// msg loop
	do
	{
	    msg = dlgEdit.GetMsg();

        // Result
	    if (dlgEdit.result == 1 || msg.qualifiers == "ctrl" && msg.event == "hotkey")
	    {
		    newTags = dlgEdit.Control("SetTags").value
		    if (!dlgEdit.result) dlgEdit.EndDlg;

		    newTags = format_tags(newTags);
		    newTags = remove_duplicate(newTags);

// Add the new tags to the selected items and restore the timestamp
		    cmd.deselect = false;
		    var modified, ADSfile, fileWrite, eItem, text, text2, textbom, myArray, ADS = "", ADS1 = "", ADS2 = "", ADS3 = "";
            for (var eSel = new Enumerator(tab.selected); !eSel.atEnd(); eSel.moveNext())
	        {
			    eItem = eSel.item();
			    modified = eItem.modify;
                ADSfile =  eItem + ":Tags";

                // If ADSfile exists, get ADS
                if (fsu.Exists(ADSfile))
                {
					var openText = fsu.OpenFile(ADSfile, "r");    // Read file
					var text = openText.Read();
					openText.Close;

                    textbom = ST.Decode(text, "utf-8 bom");       // Decode as UTF-8 with BOM
					if (textbom.trim() == "") {ADS = newTags};
					else
					{
                       myArray = textbom.split("\r\n");           // The seprator is \r\n
					   if (myArray[1]) ADS2 = myArray[1];         // Get the second line
					   if (myArray[2]) ADS3 = myArray[2];         // Get the third line
					   ADS = newTags + "\r\n" + ADS2 + "\r\n" + ADS3;
					}
                } else {ADS = newTags}

			    if (ADS.trim() == "")
			    {
				    fileWrite = fsu.OpenFile(ADSfile,"d");
					//fileWrite = fsu.GetItem(ADSfile).Open("d");
					fileWrite.Close;
			    } else
			    {
	                fileWrite = fsu.OpenFile(ADSfile, "wa");
	                if (fileWrite.error == 0)
	                {
	                    fileWrite.Write(ADS);
	                }
	                fileWrite.Close;
				}
			    modified = modified.Format("D#yyyy-MM-dd T#HH:mm:ss")
			    //cmd.Runcommand('SetAttr MODIFIED "'+ modified +'"')
			    cmd.RunCommand ('SetAttr "' + eItem + '" Meta "lastmodifieddate:' + modified + '"')
	        }
	        cmd.Runcommand("RefreshColumnADS");   // Refresh column

            if (newTags == "") return;
            var array = newTags.split(", ");
		    var tags = tab.vars.Get('DOpusTag');
		    var tagsArray = tags.split(", ");                            // Seprator
            var newTags2 = "", dupTag;

            // Remove Duplicate
		    for (i = 0; array[i]; i++)
		    {
			    dupTag = "";
		        for (ii = 0; tagsArray[ii]; ii++)
				{
				    if (array[i] == tagsArray[ii])
					{
						dupTag = 1;
					}
				}

			    if (dupTag != 1)
				{
					newTags2 += array[i] + ", ";
				}
            }

            if (newTags2 == "") return;
		    var newTags3 = newTags2 + tags;
            tab.vars.Set('DOpusTag', newTags3);

            // Write new tags to DOpusTag.dat
		    var tagFile = "%appdata%\\GPSoftware\\Directory Opus\\Scripts\\INI\\DOpusTag.dat";   // File path
		    var oFileWrite = fsu.OpenFile(tagFile, "wa");
            var utf8bom = ST.Encode(newTags3, "utf-8 bom");   // Decode as UTF-8 with BOM
            if (oFileWrite.error == 0)
            {
                oFileWrite.Write(utf8bom)
            }
            oFileWrite.Close()
	    }
	} while (msg == true)
}


function Onatbl(scriptCmdData)
{
    var dlg = scriptCmdData.func.Dlg();
    var tab = scriptCmdData.func.sourcetab;
    var cmd = scriptCmdData.func.command;
	if (!tab.selected.count) {
		DOpus.Notify("Add Tags by List", "Nothing selected!");
		return
	}

    var strName;

    dlg.title = "Directory Opus";
    dlg.template = "dlgList";
    dlg.detach = true;
    dlg.Create();
	dlg.Control("static").style = "b";
	dlg.Control("static").label = "Add Tags by List";                       // Topic
	dlg.Control("static2").label = "Tick the tags that shall be added to the tags of the selected items.";   // Notes
	dlg.Control("static3").label = Script.LoadImage("Label.png");           // Icon
	dlg.Control("static4").label = Script.LoadImage("Find.png");            // Icon

    InitTags(tab, true);
    updateList(tab, dlg, strName);
	dlg.AddHotkey("enterdlg", "ctrl+enter")                                 // Add Hotkeys
	var dlgMenu = scriptCmdData.func.Dlg();

    dlg.Show();

	var lst = dlg.Control("list");
	var col = lst.columns;
	var v_editFilter = "", v_editFilterS = "";



    while (true)
	{
        var msg = dlg.GetMsg();
        if (!msg.result) break;

// Result
        if (msg.qualifiers == "ctrl" && msg.event == "hotkey" || dlg.result == 1) {
		    var tags = "";
            for (i=0; i < lst.count; i ++)
		    {
                if (lst.GetItemAt(i).checked == 1)
				{
				    tags += lst.GetItemAt(i).name;
					tags += ", ";
                }
            }
			if (!dlg.result) dlg.EndDlg;

			// Add the checked tags to the selected items and restore the timestamp
			tags = tags.substr(0, tags.length-2);
			cmd.deselect = false;
			var modified, ADSfile, fileWrite, eItem, text, text2, textbom, myArray, ADS = "", ADS1 = "", ADS2 = "", ADS3 = "";
			
			for (var eSel = new Enumerator(tab.selected); !eSel.atEnd(); eSel.moveNext())
			{
				eItem = eSel.item();
				modified = eItem.modify;
				ADSfile =  eItem + ":Tags";
				// If ADSfile exists, get ADS
				if (fsu.Exists(ADSfile))
				{
					var openText = fsu.OpenFile(ADSfile, "r");    // Read file
					var text = openText.Read();
					openText.Close;

					textbom = ST.Decode(text, "utf-8 bom");       // Decode as UTF-8 with BOM
					if (textbom.trim() == "") {ADS = tags};
					else
					{
						myArray = textbom.split("\r\n");           // The seprator is \r\n
						if (myArray[1]) ADS2 = myArray[1];         // Get the second line
						if (myArray[2]) ADS3 = myArray[2];         // Get the third line
						ADS = tags + "\r\n" + ADS2 + "\r\n" + ADS3;
					}
				} else
					ADS = tags;

				if (ADS.trim() == "")
				{
					fileWrite = fsu.OpenFile(ADSfile, "d");
					fileWrite.Close;
				} else
				{
					fileWrite = fsu.OpenFile(ADSfile, "wa");
					if (fileWrite.error == 0)
					{
						fileWrite.Write(ADS);
					}
					fileWrite.Close;
				}
				modified = modified.Format("D#yyyy-MM-dd T#HH:mm:ss") 
				//cmd.Runcommand('SetAttr MODIFIED "'+ modified +'"')
				cmd.RunCommand ('SetAttr "' + eItem + '" Meta "lastmodifieddate:' + modified + '"')
			}
			cmd.Runcommand("RefreshColumnADS");   // Refresh column
			return
	}

// Right click list menu
        if (msg.event == "rclick" && lst.focus == true)
		{
            if (lst.value.index > -1)
			{
				dlgMenu.choices = DOpus.Create.Vector("New", "Edit", "Delete", "Clear Filter", "Refresh", "Copy", "Cancel");
				dlgMenu.menu = DOpus.Create.Vector(1, 0, 0, 0, 0, 0, 1);
            } else {
				dlgMenu.choices = DOpus.Create.Vector("OK", "Cancel");
                dlgMenu.menu = DOpus.Create.Vector(1, 1);
            }
            var menuReturn = dlgMenu.Show;

            switch (menuReturn)
			{
				case 1:
				    NewTags(scriptCmdData);
				    updateList(tab, dlg, strName);
				    break
				case 2:
				    var editTag = lst.GetItemAt(lst.value.index).name;
				    tab.vars.Set('editTag', editTag);
				    EditTag(scriptCmdData);
				    updateList(tab, dlg, strName);
				    break
				case 3:
				    var deleteTag = lst.GetItemAt(lst.value.index).name;
				    tab.vars.Set('deleteTag', deleteTag);
				    DeleteTag(scriptCmdData);
				    updateList(tab, dlg, strName);
				    break
				case 4:
                    dlg.Control("editFilter").value = "";
			        updateList(tab, dlg, strName);
				    break
				case 5:
				    updateList(tab, dlg, strName);
				    break
				case 6:
                    var item = lst.GetItemAt(lst.value.index).name;
				    DOpus.SetClip(item);
				    break
			}
        }

// Filter
		if (msg.control == "static4") {
			v_editFilter = dlg.Control("editFilter").value;
			if (v_editFilter.trim())
			    dlg.Control("editFilter").value = "";
			else
				dlg.Control("editFilter").value = v_editFilterS;
            updateList(tab, dlg, strName);
			v_editFilterS = v_editFilter;
		} else if (msg.event == "editchange" || msg.event == "click")
		{
			updateList(tab, dlg, strName);
		}
    }
}



function Onfbtl(scriptCmdData)
{

    var dlg = scriptCmdData.func.Dlg();
    var tab = scriptCmdData.func.sourcetab;
    var cmd = scriptCmdData.func.command;
/*
    if (!DOpus.version.AtLeast("12.30.1"))
    {
	    dlg.request("This script requires 12.30.1 and above!", "OK");
		return
    }
*/
    var strName;

    dlg.title = "Directory Opus";
    dlg.template = "dlgList";
    dlg.detach = true;
    dlg.Create();
	dlg.Control("static").style = "b";
	dlg.Control("static").label = "Find by Tags";                           // Topic
	dlg.Control("static2").label = "Tick the tags that shall be matched via Everyting tool.";   // Notes
	dlg.Control("static3").label = Script.LoadImage("Everything24.ico");    // Icon
	dlg.Control("static4").label = Script.LoadImage("Find.png");            // Icon
	//dlg.Control("static5").label = "D:\\Icons\\Clear.png";                  // Clear

    InitTags(tab, true);
    updateList(tab, dlg, strName);
	dlg.AddHotkey("enterdlg", "ctrl+enter")                                 // Add Hotkeys
	var dlgMenu = scriptCmdData.func.Dlg();

    dlg.Show();

	var lst = dlg.Control("list");
	var col = lst.columns;
	var v_editFilter = "", v_editFilterS = "";



    while (true)
	{
        var msg = dlg.GetMsg();
        if (!msg.result) break;

// Result
        if (msg.qualifiers == "ctrl" && msg.event == "hotkey" || dlg.result == 1)
		{
		    var tags = "";
            for (i=0; i < lst.count; i ++)
		    {
                if (lst.GetItemAt(i).checked == 1)
				{
				    tags += 'alternate-data-stream-utf-8:"';
					tags += lst.GetItemAt(i).name + '"';
					tags += "|";
                }
            }
			if (!dlg.result) dlg.EndDlg;

			if (tags)
			{
				tags = tags.substr(0, tags.length-1);
				cmd.RunCommand('Find CLEAR QUERYENGINE=everythingglobal QUERY ' + tags);
			}
			return
		}

// Right click list menu
        if (msg.event == "rclick" && lst.focus == true)
		{
            if (lst.value.index > -1)
			{
				dlgMenu.choices = DOpus.Create.Vector("New", "Edit", "Delete", "Clear Filter", "Refresh", "Copy", "Cancel");
				dlgMenu.menu = DOpus.Create.Vector(1, 0, 0, 0, 0, 0, 1);
            } else {
				dlgMenu.choices = DOpus.Create.Vector("OK", "Cancel");
                dlgMenu.menu = DOpus.Create.Vector(1, 1);
            }
            var menuReturn = dlgMenu.Show;

            switch (menuReturn)
			{
				case 1:
				    NewTags(scriptCmdData);
				    updateList(tab, dlg, strName);
				    break
				case 2:
				    var editTag = lst.GetItemAt(lst.value.index).name;
				    tab.vars.Set('editTag', editTag);
				    EditTag(scriptCmdData);
				    updateList(tab, dlg, strName);
				    break
				case 3:
				    var deleteTag = lst.GetItemAt(lst.value.index).name;
				    tab.vars.Set('deleteTag', deleteTag);
				    DeleteTag(scriptCmdData);
				    updateList(tab, dlg, strName);
				    break
				case 4:
                    dlg.Control("editFilter").value = "";
			        updateList(tab, dlg, strName);
				    break
				case 5:
				    updateList(tab, dlg, strName);
				    break
				case 6:
                    var item = lst.GetItemAt(lst.value.index).name;
				    DOpus.SetClip(item);
				    break
			}
        }

// Filter
		if (msg.control == "static4") {
			v_editFilter = dlg.Control("editFilter").value;
			if (v_editFilter.trim())
			    dlg.Control("editFilter").value = "";
			else
				dlg.Control("editFilter").value = v_editFilterS;
            updateList(tab, dlg, strName);
			v_editFilterS = v_editFilter;
		} else if (msg.event == "editchange" || msg.event == "click")
		{
			updateList(tab, dlg, strName);
		}
    }
}



function updateList(tab, dlg, strName) {
/*
    if (!DOpus.version.AtLeast("12.30.1"))
    {
	    dlg.request("This script requires 12.30.1 and above!", "OK");
		return
    }
*/
    var lst = dlg.Control("list");
	var col = lst.columns;

// Add columns
	var vecOrder = col.GetDisplayOrder();
    if (vecOrder.size < 1)
	{
	    col.AddColumn("0:Name");
	    col.AddColumn("1:Index");
	}

    lst.redraw = false;
    lst.RemoveItem(-1);

// List items
    //var text = "tag1, tag2, tag3, tag4, tag5, tag6, tag7, tag8, tag9, tag10, tag11, tag12, tag13, tag14, tag15, tag16, tag17, tag18, tag19, tag20, tag21, tag22, tag23, tag24, tag25, tag26, tag27, tag28, tag29, tag30, tag31, tag32, tag33, tag34, tag35";
	var text = tab.vars.Get('DOpusTag');
	var myArray = text.split(", ");                           // Seprator

	var idx, count = 0, itemCount = 0, col1_w;

    // Get var ADScheck
    var ADS = tab.vars.Get('ADScheck');

// Add checked list
    if (ADS && ADS != "undefined")
	{
	    var array = ADS.split(", ");                          // Seprator
		for (i = 0; array[i]; i++)
		{
		    count++;
			var checkedTag = array[i].trim();

	        var strFilter = dlg.Control("editFilter").value;   // Get filter value
	        strFilter = strFilter.toLowerCase();

		    if (checkedTag.search(strFilter) == -1 || !checkedTag) {
			    continue;
		    }
	        idx = lst.AddItem(array[i]);                     // Add each item to list
            lst.GetItemAt(idx).subitems(0) = count;          // Add number to column 1 (Index)
			lst.GetItemAt(idx).checked = 1;

            itemCount++;
			var checkedCase = 1;
			checkedTag = "";
			strFilter = "";
	    }
    }

// Add list
    var dupTag;
    for (i = 0; myArray[i]; i++)
	{
        strName = myArray[i].trim();
		var strRealName = myArray[i].trim();

        // Remove Duplicate
        dupTag = "";
		if (checkedCase == 1)
		{
			for (ii = 0; array[ii]; ii++)
			{
				if (array[ii].trim() == strName)
				{
					dupTag	= 1;
				}
			}
		}

		if (dupTag == "")
		{
			count++;
	        var strFilter = dlg.Control("editFilter").value;      // Get filter value
	        strFilter = strFilter.toLowerCase();

		    if (strRealName.search(strFilter) == -1 || !strName) {
			    continue;
		    }

            idx = lst.AddItem(strName);                           // Add each item to list
		    lst.GetItemAt(idx).subitems(0) = count;               // Add number to column 1 (Index)

            itemCount++;
		    strRealName = "";
		    strFilter = "";
		}
    }

// Set the columns display order
	if (true)
	{
	    col.SetDisplayOrder(1,0);
	}

// Resize columns based on number of lists
    //col.AutoSize();
	if (String(i).length > 2) {
		col1_w = String(i).length+45;
		col.GetColumnAt(1).width = col1_w;
	} else {
		col1_w = String(i).length+26;
	    col.GetColumnAt(1).width = col1_w;
	}

	if (itemCount > 28)
	{
	   col.GetColumnAt(0).width = 445-col1_w;   // The scrollbar takes up some width
	}
	else
	{
	   col.GetColumnAt(0).width = 462-col1_w;
	}
	lst.redraw = true;
}



function InitTags(tab, getSel) {

    // Create folder if does not exist
    if (!fsu.Exists("%appdata%\\GPSoftware\\Directory Opus\\Scripts\\INI\\")) {
	    cmd.Runcommand('CreateFolder "%appdata%\\GPSoftware\\Directory Opus\\Scripts\\INI\\"');
    }

    // Get ADS of the selected items 
    if (getSel) {
        var eItem, ADSfile, text, textbom, myArray, ADS, ADS2 = "", i = 0;

		for (var eSel = new Enumerator(tab.selected); !eSel.atEnd(); eSel.moveNext())
		{
			i++;
			ADS2 = ADS;
			eItem = eSel.item();
			ADSfile =  eItem + ":Tags";
			// If ADSfile exists
			if (fsu.Exists(ADSfile))
			{
				var openText = fsu.OpenFile(ADSfile, "r");    // Read file
				var text = openText.Read();
				openText.Close;

				textbom = ST.Decode(text, "utf-8 bom");       // Decode as UTF-8 with BOM
				myArray = textbom.split("\r\n");              // The seprator is \r\n
				ADS = myArray[0];                             // Get the first line
			}
			else {ADS = ""; break}
			if (i > 1 && ADS != ADS2) {ADS = ""; break};
		}
		ADS = format_tags(ADS);
	} else
		var ADS = "";
	tab.vars.Set('ADScheck', ADS);

    // Read the tag file DOpusTag.dat and set it to variable DOpusTag
    var tagFile = "%appdata%\\GPSoftware\\Directory Opus\\Scripts\\INI\\DOpusTag.dat";   // File path
    if (fsu.Exists(tagFile))
    {
	    var openText = fsu.OpenFile(tagFile, "r");    // Read file
	    var tags = openText.Read();
	    openText.Close;
        tags = ST.Decode(tags, "utf-8 bom");
    } else {var tags = "";}
    tab.vars.Set('DOpusTag', tags);
}



function NewTags(scriptCmdData) {

    var dlgEdit = scriptCmdData.func.Dlg();
    var tab = scriptCmdData.func.sourcetab;

    dlgEdit.title = "Directory Opus";
    dlgEdit.template = "dlgEditTags";
    dlgEdit.detach = true;
    dlgEdit.Create();

	dlgEdit.Control("static").style = "b";
	dlgEdit.Control("static").label = 'Enter new Tags';                         // Topic
	dlgEdit.Control("static2").label = 'One tag per line, or enter new tags separated by commas.';   // Notes
	dlgEdit.Control("static3").label = Script.LoadImage("Edit-new.png");        // Icon

	dlgEdit.Control("editTags").value = "";                                     // Default value
	dlgEdit.Control("editTags").SelectRange(0, -1);                             //Select Default value
	dlgEdit.AddHotkey("enterdlg", "ctrl+enter")                                 // Add Hotkeys

    dlgEdit.Show();

	var newTags, dupTag

// msg loop
	do
	{
	    msg = dlgEdit.GetMsg();

        // Result
	    if (dlgEdit.result == 1 || msg.qualifiers == "ctrl" && msg.event == "hotkey")
	    {
		    newTags = dlgEdit.Control("editTags").value
		    if (!dlgEdit.result) dlgEdit.EndDlg;

		    if (newTags.trim() == "") {return;}
            newTags = format_tags(newTags);
		    newTags = remove_duplicate(newTags);
		    var array = newTags.split(", ");                             // Seprator

		    var tags = tab.vars.Get('DOpusTag');
		    var tagsArray = tags.split(", ");                            // Seprator

            // Remove Duplicate
		    newTags = "";
		    for (i = 0; array[i]; i++)
		    {
			    dupTag = "";
		        for (ii = 0; tagsArray[ii]; ii++)
				{
				     if (array[i] == tagsArray[ii])
					 {
						 dupTag = 1;
					 }
				}

			    if (dupTag != 1)
				{
					newTags += array[i] + ", ";
				}
            }

            if (!newTags) {return};
		    var newTags2 = newTags + tags;
            tab.vars.Set('DOpusTag', newTags2);

            // Write new tags to DOpusTag.dat
		    var tagFile = "%appdata%\\GPSoftware\\Directory Opus\\Scripts\\INI\\DOpusTag.dat";   // File path
		    var oFileWrite = fsu.OpenFile(tagFile, "wa");
            var utf8bom = ST.Encode(newTags2	, "utf-8 bom");   // Decode as UTF-8 with BOM
            if (oFileWrite.error == 0)
            {
                oFileWrite.Write(utf8bom)
            }
            oFileWrite.Close()
	    }
	} while (msg == true)
}



function EditTag(scriptCmdData) {

    var dlgEdit = scriptCmdData.func.Dlg();
    var tab = scriptCmdData.func.sourcetab;

    dlgEdit.title = "Directory Opus";
    dlgEdit.template = "dlgEditTags";
    dlgEdit.detach = true;
    dlgEdit.Create();

	dlgEdit.Control("static").style = "b";
	dlgEdit.Control("static").label = 'Edit Tags';                              // Topic
	dlgEdit.Control("static2").label = 'One tag per line, or enter new tags separated by commas.';   // Notes
	dlgEdit.Control("static3").label = Script.LoadImage("Edit.png");            // Icon

	var editTag = tab.vars.Get('editTag', editTag);
	dlgEdit.Control("editTags").value = editTag;                                // Default value
	dlgEdit.Control("editTags").SelectRange(0, -1);                             //Select Default value
	dlgEdit.AddHotkey("enterdlg", "ctrl+enter")                                 // Add Hotkeys

    dlgEdit.Show();

	var newTag, newTags = "", dupTag

// msg loop
	do
	{
	    msg = dlgEdit.GetMsg();

        // Result
	    if (dlgEdit.result == 1 || msg.qualifiers == "ctrl" && msg.event == "hotkey")
	    {
		    newTag = dlgEdit.Control("editTags").value
		    if (!dlgEdit.result) dlgEdit.EndDlg;

		    if (newTag.trim() == "") {return;}
            newTag = format_tags(newTag);
		    newTag = remove_duplicate(newTag);

		    var tags = tab.vars.Get('DOpusTag');

		    // Remove duplicate
	        var tags = tab.vars.Get('DOpusTag');
		    var tagsArray = newTag.split(", ");
		    var tagsArray2 = tags.split(", ");                            // Seprator

		    for (i = 0; tagsArray2[i]; i++)
		    {
			    dupTag = "";
			    for (ii = 0; tagsArray[ii]; ii++)
				{
				     if (tagsArray2[i] == tagsArray[ii])
					 {
						 dupTag = 1;
					 }
				}
				if (dupTag != 1 && tagsArray2[i] != editTag)
				{
					newTags += tagsArray2[i] + ", ";
				}
		    }

            if (newTags == "") return;
		    newTags = newTag + ", " + newTags;
		    newTags = newTags.substr(0, newTags.length-2);
            tab.vars.Set('DOpusTag', newTags);

            // Write new tags to DOpusTag.dat
		    var tagFile = "%appdata%\\GPSoftware\\Directory Opus\\Scripts\\INI\\DOpusTag.dat";   // File path
		    var oFileWrite = fsu.OpenFile(tagFile, "wa");
            var utf8bom = ST.Encode(newTags, "utf-8 bom");   // Decode as UTF-8 with BOM
            if (oFileWrite.error == 0)
            {
                oFileWrite.Write(utf8bom)
            }
            oFileWrite.Close()
	    }
	} while (msg == true)
}



function DeleteTag(scriptCmdData) {

    var tab = scriptCmdData.func.sourcetab;
    var deleteTag = tab.vars.Get('deleteTag');
	if (deleteTag == "") return;

	var tags = tab.vars.Get('DOpusTag');
	var tagsArray = tags.split(", ");                           // Seprator
	var newTags = "";
	for (i = 0; tagsArray[i]; i++)
	{
	    if (tagsArray[i] != deleteTag)
		{
			newTags += tagsArray[i] + ", ";
        }
	}

    newTags = newTags.substr(0, newTags.length-2)
    tab.vars.Set('DOpusTag', newTags);

    // Write new tags to DOpusTag.dat
	var tagFile = "%appdata%\\GPSoftware\\Directory Opus\\Scripts\\INI\\DOpusTag.dat";   // File path
	var oFileWrite = fsu.OpenFile(tagFile, "wa");
    var utf8bom = ST.Encode(newTags, "utf-8 bom");   // Decode as UTF-8 with BOM
    if (oFileWrite.error == 0)
    {
        oFileWrite.Write(utf8bom)
    }
    oFileWrite.Close()
}



function format_tags(str) {

    var array = String(str).split(",");                                // Seprator
	var newTag1 = array[0].trim().replace(/\|/g,'');
	if (newTag1 == "") {return newTag1};

    var tag, newTags = "";
	for (i = 0; array[i]; i++)
	{
		tag = array[i].trim().replace(/\|/g,'');
		if (tag)
		{
		    newTags += tag + ", ";
		}
    }
	newTags = newTags.substr(0, newTags.length-2);

    var array2 = newTags.split("\r\n");                                // Seprator
	newTags = "";
	for (i = 0; array2[i]; i++)
	{
		tag = array2[i].trim().replace(/\|/g,'');
		if (tag)
		{
		    newTags += tag + ", ";
		}
	}
	newTags = newTags.substr(0, newTags.length-2);
	return newTags;
}



function remove_duplicate(str) {

    var array = String(str).split(", ");                               // Seprator
	var newTag1 = array[0].trim().replace(/\|/g,'');
	if (newTag1 == "") {return newTag1};

    var vecTags = DOpus.NewVector();
    for (i=0; array[i]; i++)
    {
	    vecTags.push_back = array[i];
    }

    var removedNum = vecTags.unique();
    if (removedNum == 0) {return String(str)};

    var newTags = "";
    for (var eItems = new Enumerator(vecTags); !eItems.atEnd(); eItems.moveNext() )
    {
	    newTags += eItems.item() + ", ";
    }
    newTags = newTags.substr(0, newTags.length-2);
	return newTags;
}



==SCRIPT RESOURCES
<resources>
	<resource name="dlgEditTags" type="dialog">
		<dialog fontsize="8" height="120" lang="english" resize="yes" title="Edit Tags" width="522">
			<control halign="left" height="62" multiline="yes" name="editTags" resize="wh" type="edit" width="506" x="8" y="32" />
			<control close="1" default="yes" height="14" name="btnApply" resize="xy" title="&amp;Apply" type="button" width="50" x="409" y="99" />
			<control close="0" height="14" name="btnCancel" resize="xy" title="&amp;Cancel" type="button" width="50" x="462" y="99" />
			<control halign="left" height="8" name="static" title="Edit Tags" type="static" valign="top" width="472" x="39" y="7" />
			<control halign="left" height="20" image="yes" name="static3" title="Static" type="static" valign="top" width="20" x="10" y="4" />
			<control halign="left" height="8" name="static2" title="Notes" type="static" valign="top" width="472" x="39" y="17" />
		</dialog>
	</resource>
	<resource name="dlgList" type="dialog">
		<dialog fontsize="9" height="350" lang="english" resize="yes" title="Directory Opus" width="279">
			<control halign="left" height="12" name="editFilter" resize="xy" tip="Filter" type="edit" width="118" x="21" y="331" />
			<control checkboxes="auto" fullrow="yes" height="287" name="list" noheader="yes" resize="wh" type="listview" viewmode="details" width="266" x="6" y="40" />
			<control close="1" default="yes" height="12" name="btnOK" resize="xy" title="&amp;OK" type="button" width="42" x="184" y="332" />
			<control close="0" height="12" name="btnClose" resize="xy" title="C&amp;ancel" type="button" width="42" x="229" y="332" />
			<control halign="left" height="8" name="static" title="Topic" type="static" valign="top" width="245" x="25" y="7" />
			<control halign="left" height="13" image="yes" name="static3" title="Static" type="static" valign="top" width="13" x="7" y="8" />
			<control halign="left" height="12" image="yes" name="static4" notify="yes" resize="xy" title="Static" type="static" valign="top" width="12" x="8" y="331" />
			<control halign="left" height="16" name="static2" title="Notes" type="static" valign="top" width="245" x="25" y="20" />
		</dialog>
	</resource>
	<resource name="dlgSetTags" type="dialog">
		<dialog fontsize="8" height="120" lang="english" resize="yes" title="Set Tags" width="522">
			<control halign="left" height="62" multiline="yes" name="SetTags" resize="wh" type="edit" width="506" x="8" y="32" />
			<control close="1" default="yes" height="14" name="btnOK" resize="xy" title="&amp;OK" type="button" width="50" x="409" y="99" />
			<control close="0" height="14" name="btnCancel" resize="xy" title="&amp;Cancel" type="button" width="50" x="462" y="99" />
			<control halign="left" height="8" name="static" title="Set Tags" type="static" valign="top" width="472" x="39" y="6" />
			<control halign="left" height="20" image="yes" name="static3" title="Static" type="static" valign="top" width="20" x="10" y="4" />
			<control halign="left" height="8" name="static2" title="Notes" type="static" valign="top" width="472" x="40" y="17" />
		</dialog>
	</resource>
</resources>